import re, subprocess, math, sys, functools, urllib.parse

g_res = {}
g_min = 1000000000
g_max = 0

funcs_to_plot = ['K&R', 'Bernstein', 'Meiyan', 'x65599', 'FNV-1a', 'x17 unrolled',
	'Larson', 'CRC-32', 'SBox', 'Murmur2', 'Paul Hsieh', 'lookup3', 'XXHfast32']

def parse_out(out, test_name):
	global g_res, g_min, g_max, funcs_to_plot
	for line in out.split('\r\n'):
		m = re.match(r'\s*(.+): ([0-9. ]+)', line)
		if m and m.group(1) in funcs_to_plot:
			g_res.setdefault(m.group(1), [])
			num = float( m.group(2) )
			g_res[m.group(1)].append(num)
			g_max = max(num, g_max)
			g_min = min(num, g_min)

# modified from http://code.google.com/apis/chart/docs/data_formats.html#encoding_data
def extended_encode(num_data, min_value, max_value):
	MAP = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-.'
	encoded = ''
	for num in num_data:
		scaled = int(len(MAP) * len(MAP) * (num - min_value) // (max_value - min_value))
		if scaled > len(MAP) * len(MAP) - 1:
			encoded += '..'
		elif scaled < 0:
			encoded += '__'
		else:
			encoded += MAP[scaled // len(MAP)] + MAP[scaled % len(MAP)]
	return encoded

# Read files
tests = ['dic_common_words.txt', 'dic_win32.txt', 'dic_numbers.txt',
		 'dic_prefix.txt', 'dic_postfix.txt', 'dic_variables.txt',
		 'dic_Shakespeare.txt', 'dic_fr.txt', 'dic_ip.txt']
test_names = ['Words', 'Win32', 'Numbers',
		 'Prefix', 'Postfix', 'Variables',
		 'Sonnets', 'UTF-8', 'IPv4']

for test, name in zip(tests, test_names):
	out = subprocess.check_output(['release/hash', '/q', test]).decode(sys.stdin.encoding)
	parse_out(out, name)

# Prepare chart data
encoded_x = ''
encoded_y = ''
for i, func in enumerate(funcs_to_plot):
	values = g_res[func]
	encoded_x += extended_encode( [i] * len(values), 0, len(funcs_to_plot) - 1 )
	encoded_y += extended_encode(values, g_min, g_max)

func_labels = '|'.join( map(urllib.parse.quote_plus, funcs_to_plot) )
test_labels = '|'.join( map(urllib.parse.quote_plus, test_names) )

MIN_LINE = 0.95
MAX_LINE = 1.05
min_line = (MIN_LINE - g_min) / (g_max - g_min)
max_line = (MAX_LINE - g_min) / (g_max - g_min)

# Output chart URL
print('http://chart.apis.google.com/chart?cht=s&chs=800x350' +
	'&chtt=' + urllib.parse.quote_plus('Hash function quality (using the formula from Red Dragon Book)') +
	'&chco=ff00cc|FF0000|FFFF00|336600|111ca4|FF9933|808080|33FFFF|000000' +
	'&chxt=x,y&chxl=0:|' + func_labels +
	'&chxr=1,' + str(g_min) + ',' + str(g_max) +
	'&chdl=' + test_labels +
	'&chd=e:' + encoded_x + ',' + encoded_y +
	'&chm=h,c0c0c0,0,' + '{:.4g}'.format(min_line) + ',1|' +
	'h,c0c0c0,0,' + '{:.4g}'.format(max_line) + ',1' )
